﻿using System;
using System.Windows.Forms;
using HeuristicLab.Core.Views;
using HeuristicLab.MainForm;
using HeuristicLab.Problems.BinPacking.PackingPlans;
using HeuristicLab.Problems.BinPacking.Interfaces;
using HeuristicLab.Problems.BinPacking.PackingItem;
using HeuristicLab.Problems.BinPacking.PackingBin;
using HeuristicLab.Problems.BinPacking.Dimensions;
using HeuristicLab.Core;
using HeuristicLab.Data;

namespace HeuristicLab.Problems.BinPacking.Views {
  [View("2-dimensional packing plan View")]
  [Content(typeof(RegularSimpleRotationPackingPlan<TwoDimensionalPacking, RectangularPackingBin, RectangularPackingItem>), true)]
  public partial class PackingPlan2DView : NamedItemView {

    public PackingPlan2DView() {
      InitializeComponent();
    }


    public new RegularSimpleRotationPackingPlan<TwoDimensionalPacking, RectangularPackingBin, RectangularPackingItem> Content {
      get { return (RegularSimpleRotationPackingPlan<TwoDimensionalPacking, RectangularPackingBin, RectangularPackingItem>)base.Content; }
      set { base.Content = value; }
    }


    protected override void OnContentChanged() {
      base.OnContentChanged();
      binSelection.Items.Clear();
      if (Content == null) {
        Redraw();
      } else {
        for (int i = 0; i < Content.NrOfBins; i++)
          binSelection.Items.Add(i);
        Redraw(Content);
      }
    }


    private void Redraw() {
      packingPlan2D.InitializeContainer(0, 0);
    }

    private void Redraw(RegularSimpleRotationPackingPlan<TwoDimensionalPacking, RectangularPackingBin, RectangularPackingItem> plan) {     
      int currentBin = (binSelection != null && binSelection.SelectedItem != null) ? (int)(binSelection.SelectedItem) : 0;
      var bin = plan.PackingBinMeasures;
      packingPlan2D.InitializeContainer(bin.Width, bin.Height);
      int i = 0;
      foreach (var item in plan.PackingItemMeasures) {
        var position = plan.PackingItemPositions[i++];
        if (position.AssignedBin == currentBin)
          packingPlan2D.AddItemToContainer(item.Width, item.Height, position.X, position.Y, i.ToString());
      }
    }

    private void binSelection_SelectedIndexChanged(object sender, EventArgs e) {
      Redraw(Content);
      packingPlan2D.Invalidate();
      packingPlan2D.Update();
    }

  }
}
