﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Encodings.PermutationEncoding;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Problems.BinPacking.Interfaces;

namespace HeuristicLab.Encodings.PackingEncoding.PackingSequence {
  [Item("PackingSequenceCrossover", "An operator which crosses two PackingSequence representations.")]
  [StorableClass]
  public class PackingSequenceCrossover : PackingSolutionCrossover, IPackingSequenceOperator {

    [StorableConstructor]
    protected PackingSequenceCrossover(bool deserializing) : base(deserializing) { }
    protected PackingSequenceCrossover(PackingSequenceCrossover original, Cloner cloner) : base(original, cloner) { }
    public PackingSequenceCrossover()
      : base() {
        ParentsParameter.ActualName = "PackingSequence";
        ChildParameter.ActualName = "PackingSequence";
    }
    public override IDeepCloneable Clone(Cloner cloner) {
      return new PackingSequenceCrossover(this, cloner);
    }

    public PackingSequenceEncoding Cross(IRandom random, PackingSequenceEncoding parent1, PackingSequenceEncoding parent2) {
      PackingSequenceEncoding result = new PackingSequenceEncoding();
      result.PackingSequence = CosaCrossover.Apply(random, parent1.PackingSequence, parent2.PackingSequence);
      return result;
    }

    public override IOperation Apply() {
      var parents = ParentsParameter.ActualValue;

      ChildParameter.ActualValue =
        Cross(RandomParameter.ActualValue, parents[0] as PackingSequenceEncoding, parents[1] as PackingSequenceEncoding);

      return base.Apply();
    }
  }
}
